package com.javacodegeeks.example;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;

import com.atlassian.cache.Cache;
import com.atlassian.cache.CacheLoader;
import com.atlassian.cache.CacheManager;
import com.atlassian.cache.CacheSettingsBuilder;
import com.atlassian.confluence.content.render.xhtml.ConversionContext;
import com.atlassian.confluence.macro.Macro;
import com.atlassian.confluence.macro.MacroExecutionException;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.pages.PageManager;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.plugin.spring.scanner.annotation.component.Scanned;
import com.atlassian.plugin.spring.scanner.annotation.imports.ComponentImport;

@Scanned
public class CachedSpacesMacro implements Macro {
	
	private final SpaceManager spaceManager;
	private final PageManager pageManager;
	private final Cache<String, String> cache;

	@Autowired
	public CachedSpacesMacro(@ComponentImport SpaceManager spaceManager, @ComponentImport PageManager pageManager,
			@ComponentImport CacheManager cacheManager) {
		this.spaceManager = spaceManager;
		this.pageManager = pageManager;
		this.cache = cacheManager.getCache(CachedSpacesMacro.class.getName() + ".cache", 
				new CachedSpacesMacroLoader(), 
				new CacheSettingsBuilder().expireAfterAccess(60, TimeUnit.MINUTES).build());
	}
	
	@Override
	public String execute(Map<String, String> map, String body, ConversionContext context)
			throws MacroExecutionException {
		long startTime = System.currentTimeMillis();
		String output = cache.get(CachedSpacesMacro.class.getName() + ".cache"); 
		long endTime = System.currentTimeMillis();
		
		String append = "<p>Processing time in milliseconds: " + (endTime - startTime) + "</p>";
		
		return output + append;
	}

	@Override
	public BodyType getBodyType() {
		return BodyType.NONE;
	}

	@Override
	public OutputType getOutputType() {
		return OutputType.BLOCK;
	}

	private String renderSpacesMacro() {
		List<Space> spaces = spaceManager.getAllSpaces();
		List<Page> pages = new ArrayList<Page>();
		
		StringBuilder builder = new StringBuilder();
		
		builder.append("<h4>Spaces</h4>");
		builder.append("<ul>");
		for (Space space : spaces) {
			builder.append("<li>");
			builder.append(space.getName());
			pages.addAll(pageManager.getPages(space, false));
			builder.append("</li>");
		}
		builder.append("</ul>");
		
		builder.append("<h4>Pages</h4>");
		builder.append("<ul>");
		for (Page page : pages) {
			builder.append("<li>");
			builder.append(page.getTitle());
			builder.append("</li>");
		}
		builder.append("</ul>");
		
		return builder.toString();
		
	}
	
	private class CachedSpacesMacroLoader implements CacheLoader<String, String> {

		@Override
		public String load(String key) {
			return renderSpacesMacro();
		}
	}
	
}