package com.javacodegeeks.example;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;

import com.atlassian.confluence.content.render.xhtml.ConversionContext;
import com.atlassian.confluence.macro.Macro;
import com.atlassian.confluence.macro.MacroExecutionException;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.pages.PageManager;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceManager;
import com.atlassian.plugin.spring.scanner.annotation.component.Scanned;
import com.atlassian.plugin.spring.scanner.annotation.imports.ComponentImport;

@Scanned
public class SpacesMacro implements Macro {
	
	private final SpaceManager spaceManager;
	private final PageManager pageManager;

	@Autowired
	public SpacesMacro(@ComponentImport SpaceManager spaceManager, @ComponentImport PageManager pageManager) {
		this.spaceManager = spaceManager;
		this.pageManager = pageManager;
	}
	
	@Override
	public String execute(Map<String, String> map, String body, ConversionContext context)
			throws MacroExecutionException {
		long startTime = System.currentTimeMillis();
		List<Space> spaces = spaceManager.getAllSpaces();
		List<Page> pages = new ArrayList<Page>();
		
		StringBuilder builder = new StringBuilder();
		
		builder.append("<h4>Spaces</h4>");
		builder.append("<ul>");
		for (Space space : spaces) {
			builder.append("<li>");
			builder.append(space.getName());
			pages.addAll(pageManager.getPages(space, false));
			builder.append("</li>");
		}
		builder.append("</ul>");
		
		builder.append("<h4>Pages</h4>");
		builder.append("<ul>");
		for (Page page : pages) {
			builder.append("<li>");
			builder.append(page.getTitle());
			builder.append("</li>");
		}
		builder.append("</ul>");
		
		long endTime = System.currentTimeMillis();
		
		builder.append("<p>");
		builder.append("Processing time in milliseconds: " + (endTime - startTime));
		builder.append("</p>");
		
		return builder.toString();
	}

	@Override
	public BodyType getBodyType() {
		return BodyType.NONE;
	}

	@Override
	public OutputType getOutputType() {
		return OutputType.BLOCK;
	}

}
